@@ -78,7 +78,8 @@ static void get_conventional_lattice(double lattice[3][3],
 static void set_tricli(double lattice[3][3],
                        SPGCONST double metric[3][3]);
 static void set_monocli(double lattice[3][3],
-                        SPGCONST double metric[3][3]);
+                        SPGCONST double metric[3][3],
+                        const char choice[6]);
 static void set_ortho(double lattice[3][3],
                       SPGCONST double metric[3][3]);
 static void set_tetra(double lattice[3][3],
@@ -118,8 +119,7 @@ static int search_equivalent_atom(const int atom_index,
                                   const Symmetry *symmetry,
                                   const double symprec);
 static Symmetry *
-recover_symmetry_in_original_cell(const int frame[3],
-                                  const Symmetry *prim_sym,
+recover_symmetry_in_original_cell(const Symmetry *prim_sym,
                                   SPGCONST int t_mat[3][3],
                                   SPGCONST double lattice[3][3],
                                   const int multiplicity,
@@ -139,7 +139,16 @@ get_symmetry_in_original_cell(SPGCONST int t_mat[3][3],
 static Symmetry *
 copy_symmetry_upon_lattice_points(const VecDBL *pure_trans,
                                   const Symmetry *t_sym);
-
+static void find_similar_bravais_lattice(double bravais_lattice[3][3],
+                                         double origin_shift[3],
+                                         const Symmetry *symmetry,
+                                         SPGCONST double std_lattice[3][3],
+                                         SPGCONST double orig_lattice[3][3]);
+static void measure_rigid_rotation(double rotation[3][3],
+                                   SPGCONST double bravais_lattice[3][3],
+                                   SPGCONST double std_lattice[3][3]);
+static void get_orthonormal_basis(double basis[3][3],
+                                  SPGCONST double lattice[3][3]);
 
 static SPGCONST int identity[3][3] = {
   { 1, 0, 0},
@@ -149,13 +158,14 @@ static SPGCONST int identity[3][3] = {
 
 /* Return NULL if failed */
 ExactStructure *
-ref_get_exact_structure_and_symmetry(const Cell * primitive,
+ref_get_exact_structure_and_symmetry(Spacegroup * spacegroup,
+                                     const Cell * primitive,
                                      const Cell * cell,
-                                     SPGCONST Spacegroup * spacegroup,
                                      const int * mapping_table,
                                      const double symprec)
 {
   int *std_mapping_to_primitive, *wyckoffs, *equivalent_atoms;
+  double std_lattice[3][3], rotation[3][3];
   Cell *bravais;
   Symmetry *symmetry;
   ExactStructure *exact_structure;
@@ -174,6 +184,15 @@ ref_get_exact_structure_and_symmetry(const Cell * primitive,
     goto err;
   }
 
+  get_conventional_lattice(std_lattice, spacegroup);
+  /* spacegroup->bravais_lattice is overwirten. */
+  /* spacegroup->origin_shift is overwirten. */
+  find_similar_bravais_lattice(spacegroup->bravais_lattice,
+                               spacegroup->origin_shift,
+                               symmetry,
+                               std_lattice,
+                               cell->lattice);
+
   if ((wyckoffs = (int*) malloc(sizeof(int) * cell->size)) == NULL) {
     warning_print("spglib: Memory could not be allocated.");
     goto err;
@@ -214,11 +233,16 @@ ref_get_exact_structure_and_symmetry(const Cell * primitive,
     goto err;
   }
 
+  measure_rigid_rotation(rotation,
+                         spacegroup->bravais_lattice,
+                         bravais->lattice);
+
   exact_structure->bravais = bravais;
   exact_structure->symmetry = symmetry;
   exact_structure->wyckoffs = wyckoffs;
   exact_structure->equivalent_atoms = equivalent_atoms;
   exact_structure->std_mapping_to_primitive = std_mapping_to_primitive;
+  mat_copy_matrix_d3(exact_structure->rotation, rotation);
 
   return exact_structure;
 
@@ -564,6 +588,20 @@ static void get_conventional_lattice(double lattice[3][3],
   mat_get_metric(metric, spacegroup->bravais_lattice);
 
   debug_print("bravais lattice\n");
+
+  /* printf("%f %f %f\n", */
+  /*        spacegroup->bravais_lattice[0][0], */
+  /*        spacegroup->bravais_lattice[0][1], */
+  /*        spacegroup->bravais_lattice[0][2]); */
+  /* printf("%f %f %f\n", */
+  /*        spacegroup->bravais_lattice[1][0], */
+  /*        spacegroup->bravais_lattice[1][1], */
+  /*        spacegroup->bravais_lattice[1][2]); */
+  /* printf("%f %f %f\n", */
+  /*        spacegroup->bravais_lattice[2][0], */
+  /*        spacegroup->bravais_lattice[2][1], */
+  /*        spacegroup->bravais_lattice[2][2]); */
+
   debug_print_matrix_d3(spacegroup->bravais_lattice);
   debug_print("%s\n", spacegroup->choice);
 
@@ -572,7 +610,7 @@ static void get_conventional_lattice(double lattice[3][3],
     set_tricli(lattice, metric);
     break;
   case MONOCLI: /* b-axis is the unique axis. */
-    set_monocli(lattice, metric);
+    set_monocli(lattice, metric, spacegroup->choice);
     break;
   case ORTHO:
     set_ortho(lattice, metric);
@@ -627,25 +665,52 @@ static void set_tricli(double lattice[3][3],
 }
 
 static void set_monocli(double lattice[3][3],
-                        SPGCONST double metric[3][3])
+                        SPGCONST double metric[3][3],
+                        const char choice[6])
 {
-  /* Lattice is expected to be C centring */
-  double a, b, c, beta;
+  double a, b, c, angle;
+  int pos_char;
 
   debug_print("set_monocli:\n");
   debug_print_matrix_d3(metric);
 
+  if (choice[0] == '-') {
+    pos_char = 1;
+  } else {
+    pos_char = 0;
+  }
+
   a = sqrt(metric[0][0]);
   b = sqrt(metric[1][1]);
   c = sqrt(metric[2][2]);
-  lattice[0][0] = a;
-  lattice[1][1] = b;
-  beta = acos(metric[0][2] / a / c);
-  lattice[0][2] = c * cos(beta);
-  lattice[2][2] = c * sin(beta);
 
-  debug_print("beta %f\n", beta);
-  debug_print_matrix_d3(lattice);
+  switch (choice[pos_char]) {
+  case 'a':
+    angle = acos(metric[1][2] / b / c);
+    lattice[0][2] = c;
+    lattice[1][0] = a;
+    lattice[0][1] = b * cos(angle);
+    lattice[2][1] = b * sin(angle);
+    break;
+  case 'b':
+    angle = acos(metric[0][2] / a / c);
+    lattice[0][0] = a;
+    lattice[1][1] = b;
+    lattice[0][2] = c * cos(angle);
+    lattice[2][2] = c * sin(angle);
+    break;
+  case 'c':
+    angle = acos(metric[0][1] / a / b);
+    lattice[0][1] = b;
+    lattice[1][2] = c;
+    lattice[0][0] = a * cos(angle);
+    lattice[2][0] = a * sin(angle);
+    break;
+  default:
+    warning_print("spglib: Monoclinic unique axis could not be found.");
+  }
+
+
 }
 
 static void set_ortho(double lattice[3][3],
@@ -750,7 +815,6 @@ get_refined_symmetry_operations(const Cell * cell,
                                 const double symprec)
 {
   int t_mat_int[3][3];
-  int frame[3];
   double inv_prim_lat[3][3], t_mat[3][3];
   Symmetry *conv_sym, *prim_sym, *symmetry;
 
@@ -781,10 +845,8 @@ get_refined_symmetry_operations(const Cell * cell,
   /* Input cell symmetry from primitive symmetry */
   mat_multiply_matrix_d3(t_mat, inv_prim_lat, cell->lattice);
   mat_cast_matrix_3d_to_3i(t_mat_int, t_mat);
-  get_surrounding_frame(frame, t_mat_int);
 
-  symmetry = recover_symmetry_in_original_cell(frame,
-                                               prim_sym,
+  symmetry = recover_symmetry_in_original_cell(prim_sym,
                                                t_mat_int,
                                                cell->lattice,
                                                cell->size / primitive->size,
@@ -1028,14 +1090,14 @@ static void get_corners(int corners[3][8],
 }
 
 static Symmetry *
-recover_symmetry_in_original_cell(const int frame[3],
-                                  const Symmetry *prim_sym,
+recover_symmetry_in_original_cell(const Symmetry *prim_sym,
                                   SPGCONST int t_mat[3][3],
                                   SPGCONST double lattice[3][3],
                                   const int multiplicity,
                                   const double symprec)
 {
   Symmetry *symmetry, *t_sym;
+  int frame[3];
   double inv_tmat[3][3], tmp_mat[3][3];
   VecDBL *pure_trans, *lattice_trans;
 
@@ -1044,6 +1106,7 @@ recover_symmetry_in_original_cell(const int frame[3],
   pure_trans = NULL;
   lattice_trans = NULL;
 
+  get_surrounding_frame(frame, t_mat);
   mat_cast_matrix_3i_to_3d(tmp_mat, t_mat);
   mat_inverse_matrix_d3(inv_tmat, tmp_mat, 0);
 
@@ -1270,3 +1333,98 @@ copy_symmetry_upon_lattice_points(const VecDBL *pure_trans,
 
   return symmetry;
 }
+
+static void find_similar_bravais_lattice(double bravais_lattice[3][3],
+                                         double origin_shift[3],
+                                         const Symmetry *symmetry,
+                                         SPGCONST double std_lattice[3][3],
+                                         SPGCONST double orig_lattice[3][3])
+{
+  int i, j, k;
+  double min_length2, length2, diff;
+  double tmat[3][3], inv_tmat[3][3], tmp_mat[3][3];
+  double brv_dot_tmat[3][3], rot_lat[3][3];
+
+  mat_inverse_matrix_d3(tmp_mat, bravais_lattice, 0);
+  mat_multiply_matrix_d3(tmat, tmp_mat, orig_lattice); /* P: tmat */
+  mat_inverse_matrix_d3(inv_tmat, tmat, 0);
+  mat_multiply_matrix_d3(brv_dot_tmat, bravais_lattice, tmat);
+
+  min_length2 = 0;
+  for (i = 0; i < 3; i++) {
+    for (j = 0; j < 3; j++) {
+      min_length2 += bravais_lattice[i][j] * bravais_lattice[i][j];
+    }
+  }
+
+  /* For no best match */
+  mat_copy_matrix_d3(rot_lat, bravais_lattice);
+
+  for (i = 0; i < symmetry->size; i++) {
+    if (mat_get_determinant_i3(symmetry->rot[i]) < 0) {
+      continue;
+    }
+
+    /* (a,b,c) PRP^-1 */
+    mat_multiply_matrix_di3(tmp_mat, brv_dot_tmat, symmetry->rot[i]);
+    mat_multiply_matrix_d3(tmp_mat, tmp_mat, inv_tmat);
+    length2 = 0;
+    for (j = 0; j < 3; j++) {
+      for (k = 0; k < 3; k++) {
+        diff = tmp_mat[j][k] - std_lattice[j][k];
+        length2 += diff * diff;
+      }
+    }
+    if (length2 < min_length2) {
+      mat_copy_matrix_d3(rot_lat, tmp_mat);
+      min_length2 = length2;
+    }
+  }
+
+  mat_inverse_matrix_d3(tmp_mat, rot_lat, 0);
+  mat_multiply_matrix_d3(tmp_mat, tmp_mat, bravais_lattice);
+  mat_multiply_matrix_vector_d3(origin_shift, tmp_mat, origin_shift);
+  for (i = 0; i < 3; i++) {
+    origin_shift[i] = mat_Dmod1(origin_shift[i]);
+  }
+  mat_copy_matrix_d3(bravais_lattice, rot_lat);
+}
+
+static void measure_rigid_rotation(double rotation[3][3],
+                                   SPGCONST double bravais_lattice[3][3],
+                                   SPGCONST double std_lattice[3][3])
+{
+  /* (a_s^ideal, b_s^ideal, c_s^ideal) = R(a_s, b_s, c_s) */
+  double brv_basis[3][3], std_basis[3][3], inv_brv_basis[3][3];
+
+  get_orthonormal_basis(brv_basis, bravais_lattice);
+  get_orthonormal_basis(std_basis, std_lattice);
+  mat_inverse_matrix_d3(inv_brv_basis, brv_basis, 0);
+  mat_multiply_matrix_d3(rotation, std_basis, inv_brv_basis);
+}
+
+static void get_orthonormal_basis(double basis[3][3],
+                                  SPGCONST double lattice[3][3])
+{
+  int i, j;
+  double length;
+  double basis_T[3][3], lattice_T[3][3];
+
+  mat_transpose_matrix_d3(lattice_T, lattice);
+  for (i = 0; i < 3; i++) {
+    mat_copy_vector_d3(basis_T[0], lattice_T[0]);
+    /* c' = axb */
+    mat_cross_product_d3(basis_T[2], lattice_T[0], lattice_T[1]);
+    /* b' = (axb)xa = c'xa */
+    mat_cross_product_d3(basis_T[1], basis_T[2], lattice_T[0]);
+  }
+  for (i = 0; i < 3; i++) {
+    length = sqrt(mat_norm_squared_d3(basis_T[i]));
+    for (j = 0; j < 3; j++) {
+      basis_T[i][j] /= length;
+    }
+  }
+
+  mat_transpose_matrix_d3(basis, basis_T);
+
+}
